searchNodes=[{"doc":"ConfigSmuggler is a library for converting Elixir-style configuration statements to and from string-encoded key/value pairs. Elixir (and Erlang)'s configuration system is somewhat richer than naïve environment variables, i.e., System.get_env/1 -style key/value configs alone can capture. Configs in Elixir are namespaced by app, can be arbitrarily nested, and contain Elixir-native data types like atoms, keyword lists, etc. ConfigSmuggler provides a bridge between Elixir applications and key/value configuration stores, especially those available at runtime. It makes it dead-simple to use platform-agnostic configuration systems with Elixir services. WARNING! The functions in ConfigSmuggler are not suitable for use on untrusted inputs! Code is eval ed, atoms are created, etc. Configs are considered privileged inputs, so don't worry about using ConfigSmuggler for its intended purpose. But please, never let user input anywhere near this module. You've been warned. Example iex&gt; encoded_configs = %{ ...&gt; # imagine you fetch this every 60 seconds at runtime ...&gt; &quot;elixir-logger-level&quot; =&gt; &quot;:debug&quot; , ...&gt; &quot;elixir-my_api-MyApi.Endpoint-url-port&quot; =&gt; &quot;8888&quot; , ...&gt; } iex&gt; ConfigSmuggler . apply ( encoded_configs ) iex&gt; Application . get_env ( :logger , :level ) :debug iex&gt; ConfigSmuggler . encode ( [ my_api : Application . get_all_env ( :my_api ) ] ) { :ok , %{ &quot;elixir-my_api-MyApi.Endpoint-url-port&quot; =&gt; &quot;8888&quot; } } Overview apply/1 applies encoded or decoded configs to the current environment. decode/1 converts an encoded config map into Elixir-native decoded configs, also returning a list of zero or more encoded key/value pairs that could not be decoded. encode/1 converts Elixir-native decoded configs (i.e., a keyword list with app name as key and keyword list of configs as value) into an encoded config map. encode_file/1 converts an entire config.exs -style file (along with all files included with Mix.Config.import_config/1 ) into an encoded config map. encode_statement/1 converts a single config statement from a config.exs -style file into an encoded config map. At the command line, mix smuggle encode &lt;filename.exs&gt; encodes a file with encode_file/1 and emits a JSON object as output. Encoding Scheme The encoded key begins with elixir and is a hyphen-separated &quot;path&quot; of atoms and modules leading to the config value we wish to set. The value is any valid Elixir term, encoded using normal Elixir syntax. Encoding is performed by Kernel.inspect/2 . Decoding is performed by Code.eval_string/1 and String.to_atom/1 . See Also If you build and deploy Erlang releases, and you want to apply encoded configs before any other apps have started, look into Distillery config providers . This feature allows specified modules to make environment changes with Application.put_env/3 , after which these changes are persisted to the release's sys.config file and the release is started normally. Gotchas Atoms and modules are expected to follow standard Elixir convention, namely that atoms begin with a lowercase letter, modules begin with an uppercase letter, and neither contains any hyphen characters. If a config file or statement makes reference to Mix.env() , the current Mix env will be substituted. This may be different than what the config file intended. Authorship and License Copyright 2019, Appcues, Inc. ConfigSmuggler is released under the MIT License .","ref":"ConfigSmuggler.html","title":"ConfigSmuggler","type":"module"},{"doc":"Applies the given config to the current environment (i.e., calls Application.put_env/3 a bunch of times). Accepts Elixir- native decoded configs or encoded config maps. iex&gt; ConfigSmuggler . apply ( [ my_app : [ foo : 22 ] ] ) iex&gt; Application . get_env ( :my_app , :foo ) 22 iex&gt; ConfigSmuggler . apply ( %{ &quot;elixir-my_app-bar&quot; =&gt; &quot;33&quot; } ) iex&gt; Application . get_env ( :my_app , :bar ) 33","ref":"ConfigSmuggler.html#apply/1","title":"ConfigSmuggler.apply/1","type":"function"},{"doc":"Decodes a map of string-encoded key/value pairs into a keyword list of Elixir configs, keyed by app. Also returns a list of zero or more invalid key/value pairs along with their errors. iex&gt; ConfigSmuggler . decode ( %{ ...&gt; &quot;elixir-my_app-some_key&quot; =&gt; &quot;22&quot; , ...&gt; &quot;elixir-my_app-MyApp.Endpoint-url-host&quot; =&gt; &quot; \\&quot; localhost \\&quot; &quot; , ...&gt; &quot;elixir-logger-level&quot; =&gt; &quot;:info&quot; , ...&gt; &quot;elixir-my_app-MyApp.Endpoint-url-port&quot; =&gt; &quot;4444&quot; , ...&gt; &quot;bad key&quot; =&gt; &quot;22&quot; , ...&gt; &quot;elixir-my_app-foo&quot; =&gt; &quot;bogus value&quot; , ...&gt; } ) { :ok , [ my_app : [ { :some_key , 22 } , { MyApp.Endpoint , [ url : [ port : 4444 , host : &quot;localhost&quot; , ] ] } , ] , logger : [ level : :info , ] , ] , [ { { &quot;elixir-my_app-foo&quot; , &quot;bogus value&quot; } , :bad_value } , { { &quot;bad key&quot; , &quot;22&quot; } , :bad_key } , ] }","ref":"ConfigSmuggler.html#decode/1","title":"ConfigSmuggler.decode/1","type":"function"},{"doc":"Converts Elixir-native decoded configs (i.e., a keyword list with app name as key and keyword list of configs as value) into an encoded config map. iex&gt; ConfigSmuggler . encode ( [ logger : [ level : :info ] , my_app : [ key : &quot;value&quot; ] ] ) { :ok , %{ &quot;elixir-logger-level&quot; =&gt; &quot;:info&quot; , &quot;elixir-my_app-key&quot; =&gt; &quot; \\&quot; value \\&quot; &quot; , } }","ref":"ConfigSmuggler.html#encode/1","title":"ConfigSmuggler.encode/1","type":"function"},{"doc":"Reads a config file and returns a map of encoded key/value pairs representing the configuration. Respects Mix.Config.import_config/1 . iex&gt; ConfigSmuggler . encode_file ( &quot;config/config.exs&quot; ) { :ok , %{ &quot;elixir-logger-level&quot; =&gt; &quot;:info&quot; , # ... } }","ref":"ConfigSmuggler.html#encode_file/1","title":"ConfigSmuggler.encode_file/1","type":"function"},{"doc":"Encodes a single Mix.Config.config/2 or Mix.Config.config/3 statement into one or more encoded key/value pairs. iex&gt; ConfigSmuggler . encode_statement ( &quot;config :my_app, key1: :value1, key2: \\&quot; value2 \\&quot; &quot; ) { :ok , %{ &quot;elixir-my_app-key1&quot; =&gt; &quot;:value1&quot; , &quot;elixir-my_app-key2&quot; =&gt; &quot; \\&quot; value2 \\&quot; &quot; , } } iex&gt; ConfigSmuggler . encode_statement ( &quot;config :my_app, MyApp.Endpoint, url: [host: \\&quot; localhost \\&quot; , port: 4444]&quot; ) { :ok , %{ &quot;elixir-my_app-MyApp.Endpoint-url-host&quot; =&gt; &quot; \\&quot; localhost \\&quot; &quot; , &quot;elixir-my_app-MyApp.Endpoint-url-port&quot; =&gt; &quot;4444&quot; , } }","ref":"ConfigSmuggler.html#encode_statement/1","title":"ConfigSmuggler.encode_statement/1","type":"function"},{"doc":"","ref":"ConfigSmuggler.html#t:decoded_configs/0","title":"ConfigSmuggler.decoded_configs/0","type":"type"},{"doc":"","ref":"ConfigSmuggler.html#t:encoded_config_map/0","title":"ConfigSmuggler.encoded_config_map/0","type":"type"},{"doc":"","ref":"ConfigSmuggler.html#t:encoded_key/0","title":"ConfigSmuggler.encoded_key/0","type":"type"},{"doc":"","ref":"ConfigSmuggler.html#t:encoded_value/0","title":"ConfigSmuggler.encoded_value/0","type":"type"},{"doc":"","ref":"ConfigSmuggler.html#t:error_reason/0","title":"ConfigSmuggler.error_reason/0","type":"type"},{"doc":"","ref":"ConfigSmuggler.html#t:validation_error/0","title":"ConfigSmuggler.validation_error/0","type":"type"},{"doc":"ConfigSmuggler provides the mix smuggle task, which encodes config.exs -style files into JSON-formatted key/value pairs. Usage: mix smuggle encode &lt;filename.exs&gt;","ref":"Mix.Tasks.Smuggle.html","title":"Mix.Tasks.Smuggle","type":"task"}]